/**
 * The MIT License (MIT)
 * Copyright (c) 2017-present Dmitry Soshnikov <dmitry.soshnikov@gmail.com>
 */

'use strict';

/**
 * Epsilon, the empty string.
 */
const EPSILON = 'ε';

/**
 * Epsilon-closure.
 */
const EPSILON_CLOSURE = `${EPSILON}*`;

module.exports = {
  EPSILON,
  EPSILON_CLOSURE,
};